@@ -5,7 +5,7 @@
 #' Functions \code{html_book()} and \code{tufte_html_book()} are simple wrapper
 #' functions of \code{html_chapter()} using a specific base output format.
 #' @inheritParams pdf_book
-#' @param toc,number_sections,fig_caption,lib_dir,template See
+#' @param toc,number_sections,fig_caption,lib_dir,template,pandoc_args See
 #'   \code{rmarkdown::\link{html_document}},
 #'   \code{tufte::\link[tufte]{tufte_html}}, or the documentation of the
 #'   \code{base_format} function.
@@ -50,7 +50,7 @@
 #' @export
 html_chapters = function(
   toc = TRUE, number_sections = TRUE, fig_caption = TRUE, lib_dir = 'libs',
-  template = bookdown_file('templates/default.html'), ...,
+  template = bookdown_file('templates/default.html'), pandoc_args = NULL, ...,
   base_format = rmarkdown::html_document, split_bib = TRUE, page_builder = build_chapter,
   split_by = c('section+number', 'section', 'chapter+number', 'chapter', 'rmd', 'none')
 ) {
@@ -58,8 +58,11 @@ html_chapters = function(
   config = base_format(
     toc = toc, number_sections = number_sections, fig_caption = fig_caption,
     self_contained = FALSE, lib_dir = lib_dir,
-    template = template, ...
+    template = template, pandoc_args = pandoc_args2(pandoc_args), ...
   )
+  if (pandoc2.0()) {
+    config$pandoc$to = 'html4'; config$pandoc$ext = '.html'
+  }
   split_by = match.arg(split_by)
   post = config$post_processor  # in case a post processor have been defined
   config$post_processor = function(metadata, input, output, clean, verbose) {
@@ -74,6 +77,12 @@ html_chapters = function(
   config
 }
 
+# add --wrap=preserve to pandoc args for pandoc 2.0:
+# https://github.com/rstudio/bookdown/issues/504
+pandoc_args2 = function(args) {
+  if (pandoc2.0()) c('--wrap', 'preserve', args) else args
+}
+
 #' @rdname html_chapters
 #' @export
 html_book = function(...) {
@@ -115,18 +124,23 @@ tufte_html_book = function(...) {
 #' @references \url{https://bookdown.org/yihui/bookdown/}
 #' @export
 html_document2 = function(
-  ..., number_sections = TRUE, base_format = rmarkdown::html_document
+  ..., number_sections = TRUE, pandoc_args = NULL, base_format = rmarkdown::html_document
 ) {
   base_format = get_base_format(base_format)
-  config = base_format(..., number_sections = number_sections)
+  config = base_format(
+    ..., number_sections = number_sections, pandoc_args = pandoc_args2(pandoc_args)
+  )
+  if (pandoc2.0()) {
+    config$pandoc$to = 'html4'; config$pandoc$ext = '.html'
+  }
   post = config$post_processor  # in case a post processor have been defined
   config$post_processor = function(metadata, input, output, clean, verbose) {
     if (is.function(post)) output = post(metadata, input, output, clean, verbose)
-    x = readUTF8(output)
+    x = read_utf8(output)
     x = restore_appendix_html(x, remove = FALSE)
     x = restore_part_html(x, remove = FALSE)
     x = resolve_refs_html(x, global = !number_sections)
-    writeUTF8(x, output)
+    write_utf8(x, output)
     output
   }
   config$bookdown_output_format = 'html'
@@ -197,7 +211,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
 
   if (!(split_level %in% 0:2)) stop('split_level must be 0, 1, or 2')
 
-  x = readUTF8(output)
+  x = read_utf8(output)
 
   i1 = find_token(x, '<!--bookdown:title:start-->')
   i2 = find_token(x, '<!--bookdown:title:end-->')
@@ -207,7 +221,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
   i6 = find_token(x, '<!--bookdown:body:end-->')
 
   r_chap = '^<!--chapter:end:(.+)-->$'
-  n = length( grep(r_chap, x) )
+  n = length(grep(r_chap, x))
 
   # Need to take care of the div tags here before restore_part_html and
   # restore_appendix_html erase the section ids of the hidden PART or APPENDIX
@@ -231,11 +245,17 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
       i = n12 == 'h1' & c('h1', head(n12, -1)) == 'h2'
       if (any(i) && n12[1] == 'h2') i[which(n12 == 'h1')[1]] = FALSE
       i = h12[i] - 1
-      if (tail(n12, 1) == 'h2' && any(n12 == 'h1')) i = c(i, length(x))
+      # need to comment out the </div> corresponding to the last <h1> in the body
+      if (tail(n12, 1) == 'h2' && any(n12 == 'h1')) {
+        for (j in (i6 - 1):(tail(h12, 1))) {
+          # the line j should close h1, and j - 1 should close h2
+          if (all(x[j - 0:1] == '</div>')) break
+        }
+        i = c(i, j)
+      }
       for (j in i) {
-        # the i-th lines should be the closing </div> for h1, or empty
-        # because of the appendix h1, which will be removed
-        if (!x[j] %in% c('</div>', '')) warning(
+        # the i-th lines should be the closing </div> for h1
+        if (x[j] != '</div>') stop(
           'Something wrong with the HTML output. The line ', x[j],
           ' is supposed to be </div>'
         )
@@ -252,7 +272,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
   if (any(c(i1, i2, i3, i4, i5, i6) == 0)) {
     x = resolve_refs_html(x, !number_sections)
     x = add_chapter_prefix(x)
-    writeUTF8(x, output)
+    write_utf8(x, output)
     return(output)
   }
 
@@ -278,7 +298,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
   if (split_level == 0) {
     html_body[idx] = ''  # remove chapter tokens
     html_body = add_chapter_prefix(html_body)
-    writeUTF8(build(
+    write_utf8(build(
       html_head, html_toc, c(html_title, html_body), NULL, NULL, NULL, output, html_foot, ...
     ), output)
     return(move_to_output_dir(output))
@@ -365,7 +385,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
       if (length(nms_chaps)) nms_chaps[i],
       nms[i], html_foot, ...
     )
-    writeUTF8(html, nms[i])
+    write_utf8(html, nms[i])
   }
   nms = move_to_output_dir(nms)
 
@@ -826,8 +846,8 @@ parse_footnotes = function(x) {
   j = which(x == '</div>')
   j = min(j[j > i])
   n = length(x)
-  r = '<li id="fn([0-9]+)"><p>.+?<a href="#fnref\\1">.</a></p></li>'
-  s = paste(x[i:n], collapse = '')
+  r = '<li id="fn([0-9]+)"><p>.+?<a href="#fnref\\1"[^>]*?>.</a></p></li>'
+  s = paste(x[i:n], collapse = '\n')
   items = unlist(regmatches(s, gregexpr(r, s)))
   list(items = setNames(items, gsub(r, 'fn\\1', items)), range = i:j)
 }
@@ -867,7 +887,7 @@ number_appendix = function(x, i1, i2, type = c('toc', 'header')) {
 # detect and move files to the output directory (if specified)
 move_files_html = function(output, lib_dir) {
   if (is.null(o <- opts$get('output_dir'))) return()
-  x = readUTF8(output)
+  x = read_utf8(output)
   # detect local resources used in HTML
   r = '[- ](src|href)="([^"]+)"'
   m = gregexpr(r, x)
@@ -879,7 +899,7 @@ move_files_html = function(output, lib_dir) {
   # detect resources in CSS
   css = lapply(grep('[.]css$', f, ignore.case = TRUE, value = TRUE), function(z) {
     d = dirname(z)
-    z = readUTF8(z)
+    z = read_utf8(z)
     r = 'url\\("?([^")]+)"?\\)'
     lapply(regmatches(z, gregexpr(r, z)), function(s) {
       s = local_resources(gsub(r, '\\1', s))
@@ -900,7 +920,7 @@ move_files_html = function(output, lib_dir) {
   f = unique(f[file_test('-f', f)])
   lapply(file.path(o, setdiff(dirname(f), '.')), dir_create)
   f2 = file.path(o, f)
-  i = !same_path(f, f2, mustWork = FALSE)
+  i = !same_path(f, f2)
   if (any(i)) file.copy(f[i], f2[i], overwrite = TRUE)
   # should not need the lib dir any more
   if (length(lib_dir) == 1 && is.character(lib_dir))
